#include <cstdio>
#include <ctime>

#include <string>

#include "hardware/pio.h"
#include "hardware/rtc.h"

#include "pico/cyw43_arch.h"
#include "pico/stdlib.h"
#include "pico/util/datetime.h"

#include "lwip/apps/http_client.h"

#include "ntp_client.h"
#include "ws2812.h"

using std::string;

#define WS2812_PIN 22

int wifi_setup(uint32_t country, const string &ssid, const string &pw, bool firstTry = true) {
    if (firstTry) {
        if (cyw43_arch_init_with_country(country)) {
            return 1;
        }
    }

    cyw43_arch_enable_sta_mode();

    netif_set_hostname(netif_default, "AbfallPicoW");

    if (cyw43_arch_wifi_connect_async(ssid.c_str(), pw.c_str(), CYW43_AUTH_WPA2_MIXED_PSK)) {
        return 2;
    }

    int flashrate = 1000;
    int status = CYW43_LINK_UP + 1;

    while (status >= 0 && status != CYW43_LINK_UP) {
        int status_new = cyw43_tcpip_link_status(&cyw43_state, CYW43_ITF_STA);

        if (status_new != status) {
            status = status_new;
            if (status < 0) {
                continue;
            }
            flashrate = flashrate / (status + 1);
            // printf("Connect status: %d %d\n", status, flashrate);
        }
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 1);
        sleep_ms(flashrate);
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 0);
        sleep_ms(flashrate);
    }

    if (status < 0) {
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 0);
    } else {
        cyw43_arch_gpio_put(CYW43_WL_GPIO_LED_PIN, 1);

        printf("IP: %s\n", ip4addr_ntoa(netif_ip_addr4(netif_default)));
    }

    return status;
}

char myBuffer[2048];

void result_callback(void *arg, httpc_result_t httpc_result, u32_t rx_content_len, u32_t srv_res,
                     err_t err) {
    printf("Transfer complete\n");
    printf("Local result=%d\n", httpc_result);
    printf("Http result=%d\n", srv_res);
}

err_t headers_callback(httpc_state_t *connection, void *arg, struct pbuf *hdr, u16_t hdr_len,
                       u32_t content_len) {
    printf("Headers received\n");
    printf("Content length=%d\n", content_len);
    printf("Header length=%d\n", hdr_len);

    pbuf_copy_partial(hdr, myBuffer, hdr->tot_len, 0);
    printf("Headers \n");
    printf("%s", myBuffer);

    return ERR_OK;
}

err_t body_callback(void *arg, struct altcp_pcb *conn, struct pbuf *p, err_t err) {
    // printf("Body\n");
    pbuf_copy_partial(p, myBuffer, p->tot_len, 0);
    // printf("%s", myBuffer);

    // TODO: Parse CSV

    return ERR_OK;
}



int main() {
    const string ssid{"Apis cerana"};
    const string pw{"2JkJEh2vptVT"};
    const uint32_t country{CYW43_COUNTRY_GERMANY};

    stdio_init_all();
    rtc_init();

    WS2812 led(WS2812_PIN);

    bool firstTry = true;
    int res = -1;
    do {
        if (firstTry) {
            res = wifi_setup(country, ssid, pw, true);
            firstTry = false;
        } else {
            printf("Setting up connection failed. Trying again after 5 sec...\n");
            sleep_ms(5000);
            res = wifi_setup(country, ssid, pw, false);
        }

    } while (res != CYW43_LINK_UP);

    uint16_t port = 80;
    httpc_connection_t settings;
    settings.result_fn = nullptr;
    settings.headers_done_fn = nullptr;

    err_t err = httpc_get_file_dns("beenas.brodbeck-online.de", port, "/abfall/abfall.csv",
                                   &settings, body_callback, nullptr, nullptr);
    // printf("Status %d\n", err);

    NtpClient::setDateTime();

    char datetime_buf[256];
    char *datetime_str = &datetime_buf[0];
    datetime_t dt;

    while (true) {
        rtc_get_datetime(&dt);
        datetime_to_str(datetime_str, sizeof(datetime_buf), &dt);
        // printf("DateTime: %s\n", datetime_str);
        led.switchColor(Color::RED);
        sleep_ms(5000);
    }

    cyw43_arch_deinit();
}
